import { router } from '../../modules/router.ts';
import { Route } from '@abearxiong/router';
import { RouterCodeModel, RouterCode } from '../../models/code.ts';

export enum CodeStatus {
  running = 'running',
  stop = 'stop',
  fail = 'fail',
}

export type CodeManager = {
  fn?: any;
  status?: CodeStatus;
  errorMsg?: string;
} & Partial<RouterCode>;
const codeDemoRun = `async function run(ctx) {
  ctx.body = 'test js';
  return ctx;
}`;
export const loadOne = async (item: RouterCodeModel) => {
  const { path, key, id, code, project } = item.toJSON();
  try {
    const fn: any = new Function(
      'ctx',
      `
    ${code}
    return run(ctx);
  `,
    );
    // run code
    const codeRunRoute = new Route(path, key, { id });
    codeRunRoute.run = fn;
    router.add(codeRunRoute);
    return {
      path,
      key,
      id,
      project,
      fn,
      status: CodeStatus.running,
    };
  } catch (e) {
    console.error('error id:', id, '\n', e);
    return {
      path,
      key,
      id,
      project,
      status: CodeStatus.fail,
      errorMsg: e.message.toString(),
    };
  }
};
export const load = async function () {
  const codes = await RouterCodeModel.findAll();
  const codeManager: CodeManager[] = codes.map((item) => {
    const { path, key, id, code, project, active } = item.toJSON();
    if (!active) {
      return {
        path,
        key,
        id,
        code,
        project,
        status: CodeStatus.stop,
      };
    }
    try {
      const fn: any = new Function(
        'ctx',
        `
      ${code}
      return run(ctx);
    `,
      );
      // run code
      const codeRunRoute = new Route(path, key, { id });
      codeRunRoute.run = fn;
      router.add(codeRunRoute);
      return {
        path,
        key,
        id,
        code,
        project,
        type: item.type,
        fn,
        status: CodeStatus.running,
      };
    } catch (e) {
      console.error('error id:', id, '\n', e);
      return {
        path,
        key,
        id,
        code,
        project,
        type: item.type,
        status: CodeStatus.fail,
        errorMsg: e.message.toString(),
      };
    }
  });
  return codeManager;
};
